/*
 * Demo.h
 *
 * Created: 19.01.2015 11:07:53
 *  Author: HL
 */ 


#ifndef DEMO_H_
#define DEMO_H_

// pin enum
typedef enum
{
	P0 = 0x01,
	P1 = 0x02,
	P2 = 0x04,
	P3 = 0x08,
	P4 = 0x10,
	P5 = 0x20,
	P6 = 0x40,
	P7 = 0x80
} PIN;


// Zugriff auf SFR Register
class SfrRegAccess
{
	public:
	static bool Get(const uint8_t reg, uint8_t bit) {
		return  (*reinterpret_cast<volatile uint8_t*>(reg + __SFR_OFFSET) & bit ) != 0;
	};
	
	static void Set(const uint8_t reg, uint8_t bit, bool value) {
		if ( value)
		*reinterpret_cast<volatile uint8_t*>(reg + __SFR_OFFSET) |=  bit;
		else
		*reinterpret_cast<volatile uint8_t*>(reg + __SFR_OFFSET) &= ~ bit;
	};

	static void Toggle(const uint8_t reg, uint8_t bit)
	{
		*reinterpret_cast<volatile uint8_t*>(reg + __SFR_OFFSET) ^= bit;
	};
};


// AVR Port
template< uint8_t sfr_pin, uint8_t sfr_ddr, uint8_t sfr_port>
struct AvrPort
{
	static const uint8_t in       = sfr_pin;
	static const uint8_t dir      = sfr_ddr;
	static const uint8_t out      = sfr_port;	
};


/// ATtiny2313 port
typedef AvrPort<0x16,0x17,0x18> PORT_B;

// digitaler eingang
template< class regs, PIN pin >
class DigitalInput 
{
	 
	public:
	DigitalInput()	{
		// { port::SetDir(pin, false); };  default
	};
	
	bool Get(void)  { return SfrRegAccess::Get(regs::in,pin); };
	bool IsSet(void)  { return SfrRegAccess::Get(regs::in,pin); };
	bool IsNotSet(void)  { return ! SfrRegAccess::Get(regs::in,pin); };
		
};

// digitaler ausgang
template< class  regs, PIN pin >
class DigitalOutput 
{
	public:
	DigitalOutput()
	{
		SfrRegAccess::Set(regs::dir,pin, true);
	};
	
	void Set(bool value = true)  { SfrRegAccess::Set(regs::out,pin, value); };
	void Clear(void)  { SfrRegAccess::Set(regs::out,pin, false); };
	void Toggle(void)  { SfrRegAccess::Toggle(regs::out); };		
};


#endif /* DEMO_H_ */